/*
 *  Kylin security execute control and Box module（模块，包功能介绍）
 *
 *  This file contains the definitions needed for Kylin security exectl
 *  functions
 *
 *  Authors:  WKL <wangkanglong@kylinos.cn>
 *
 *  Copyright (C) 2017,Tianjin KYLIN Information Technology Co., Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 */

#ifndef KYSEC_DEVCTL_H
#define KYSEC_DEVCTL_H

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>

#ifdef __cplusplus
extern "C" {
#endif

#define SSID_MAX 256
#define DNAME_MAX 128
#define DEVNAME_MAX 30
#define DISPLAY_MAX 256

#define KYSEC_DEVCTL_XML_PATH "/etc/kysec/devctl/devctl.xml"
#define KYSEC_PROPATH_XML_PATH "/etc/kysec/devctl/ppro.xml"
#define DEVCTL_SYSFILE_PATH "/sys/kernel/security/kysec/device_info"
#define DEVICE_RECORD_DB_PATH "/etc/kysec/db/whlist.db"

enum kysec_dev_type {
    KYSEC_USB = 1,
    KYSEC_SERIAL,
    KYSEC_PARALLEL,
    KYSEC_TYPEC,
    KYSEC_CDROM,
    KYSEC_PRINTER,
    KYSEC_WIRELESS,    //DEV_NET_CARD
    KYSEC_ETHERNET,   //DEV_NET_CARD
    KYSEC_BLUETOOTH,
    KYSEC_CAMERA,     //DEV_IMAGE
    KYSEC_SCANNER,    //DEV_IMAGE
    KYSEC_AUDIO,
    KYSEC_IEEE1394,  //新增
    KYSEC_PCMCIA,
    KYSEC_PCIE,
    KYSEC_SATA,
    KYSEC_PS2,
    KYSEC_VGA,
    KYSEC_HDMI,
    KYSEC_DP,
    KYSEC_INFRARED,
    KYSEC_MODEM,
    KYSEC_MMC,      //DEV_STORAGE_DRIVE
    KYSEC_SCSI_RAID,
    KYSEC_FLOPPY_DRIVE,
    KYSEC_VIDEO,
    KYSEC_DIGITAL_CAMERA,   //DEV_IMAGE
    KYSEC_MULTI_SERIAL_CARD,
    KYSEC_WINCE,
    KYSEC_DEV_MAX
};

//usb设备的类型
enum kysec_usb_type {
    KYSEC_HARD_DRIVE = 1,
    KYSEC_USB_CDROM,
    KYSEC_USB_PRINTER,
    KYSEC_USB_CAMERA,  //USB_CLASS_VIDEO
    KYSEC_USB_BLUETOOTH,  //USB_CLASS_WIRELESS_CONTROLLER,
    KYSEC_USB_STORAGE,   //USB_CLASS_MASS_STORAGE, DEV_STORAGE_DRIVE
    KYSEC_USB_PER_INTERFACE,  //新增
    KYSEC_USB_AUDIO,
    KYSEC_USB_COMM,
    KYSEC_USB_HID,
    KYSEC_USB_PHYSICAL,
    KYSEC_USB_STILL_IMAGE,
    KYSEC_USB_HUB,
    KYSEC_USB_CDC_DATA,
    KYSEC_USB_CSCID,
    KYSEC_USB_CONTENT_SEC,
    KYSEC_USB_MISC,
    KYSEC_USB_APP_SPEC,
    KYSEC_USB_VENDOR_SPEC,
    KYSEC_USB_MAX
};

struct kysec_devctl_kernel {
	int type;
	int usb_subtype;
	unsigned int pid;
	unsigned int vid;
	int perm;
	char serial[DNAME_MAX];
};
struct kysec_devctl_info {
	int type;
	int usb_subtype;
	unsigned int pid;
	unsigned int vid;
	int perm;
	int oldperm;
	char serial[DNAME_MAX];
	int special_type;
	char displayname[DISPLAY_MAX];
};
struct kysec_devctl_ssid {
	char blkssid[SSID_MAX];
};

struct kysec_ppro_path {
	char ppath[SSID_MAX];
};
struct kysec_devctl_sysf {
	int dtype;
	unsigned int devpid;
	unsigned int devvid;
	int devperm;
	char product[DNAME_MAX];
	char manufacturer[DNAME_MAX];
	char serial[DNAME_MAX];
	char dev_name[DEVNAME_MAX];
	char displayname[DISPLAY_MAX];
};

struct device_record {
	int type;
	int perm;
	unsigned int pid;
	unsigned int vid;
	char serial[DNAME_MAX];
	char access_t[DNAME_MAX];
	char duration[20];
	char displayname[DISPLAY_MAX];
};

struct kysec_devctl_blacklist {
	char name[SSID_MAX];
	int type;
	int perm;
};

/**
 * @name: kysec_devctl_add_perm
 * @Descripttion: 添加设备/接口的权限
 * @msg:
 * @param {struct kysec_devctl_info} * kysec_devctl_info结构体
 * @return 0:成功，-1:出错
 */
int kysec_devctl_add_perm(int dtype, int utype, unsigned int dpid, unsigned int dvid,
		int special_type, char *serial, char *display_name, int dperm);


/**
 * @name: kysec_devctl_remove_perm
 * @Descripttion: 删除设备/接口的权限
 * @msg:
 * @param {struct kysec_devctl_info} * kysec_devctl_info结构体
 * @return 0:成功，-1:出错
 */
int kysec_devctl_remove_perm(int dtype, int utype, unsigned int dpid, unsigned int dvid, char *serial, int dperm);

/**
 * @name: kysec_devctl_change_perm
 * @Descripttion: 修改设备/接口的权限
 * @msg:
 * @param {struct kysec_devctl_info} * kysec_devctl_info结构体
 * @return 0:成功，-1：出错
 */
int kysec_devctl_update_perm(int dtype, int utype, unsigned int dpid, unsigned int dvid, char *serial, char *display_name, int dperm);

/**
 * @name: kysec_devctl_wireless_ctl_add
 * @Descripttion: 添加无线网络热点名称到黑名单
 * @msg:
 * @param {const char} *  热点名称
 * @return 0:成功，-1：出错
 */
int kysec_devctl_wireless_ctl_add(char * ssid);

/**
 * @name:  kysec_devctl_wireless_ctl_del
 * @Descripttion: 删除黑名单中的无线网络热点
 * @msg:
 * @param {const char} *    热点名称
 * @return {*}  0:成功，-1：出错
 */
int kysec_devctl_wireless_ctl_del(char * ssid);

/**
 * @Descripttion: 批量添加设备/接口的权限
 * @data： 设备信息
 * @count: 添加个数
 * @return 0:成功，-1:出错
 */
int kysec_devctl_batch_add_perm(struct kysec_devctl_info *data, int count);

/**
 * @name: kysec_devctl_read_all_dev_data
 * @Descripttion: 获取所有设备的权限
 * @msg:
 * @param {int} *count  添加的设备个数
 * @return {*}
 */
struct kysec_devctl_info * kysec_devctl_read_all_dev_data(int *count);


/**
 * @name: kysec_devctl_read_all_ssid_data
 * @Descripttion: 获取所有的热点黑名单
 * @msg:
 * @param {int} *count  黑名单的个数
 * @return {*}
 */
struct kysec_devctl_ssid * kysec_devctl_read_all_ssid_data(int *count);

/**
 * @name:  kysec_devctl_read_not_special_data
 * @Descripttion: 获取非指定设备的权限
 * @msg: 
 * @param {int} *count
 * @return {*}
 */
struct kysec_devctl_info * kysec_devctl_read_not_special_data(int *count);

/**
 * @name: kysec_devctl_read_special_xmldata
 * @Descripttion: 获取special设备的设备名和权限
 * @msg: 
 * @param {int} *count special 设备的个数
 * @return {*}
 */
struct kysec_devctl_info *kysec_devctl_read_special_xmldata(int *count);


/**
 * @name: kysec_devctl_read_sys_devinfo
 * @Descripttion: 读取sys下的设备信息文件
 * @msg: 
 * @param {int} *count  个数
 * @return {*}
 */
struct kysec_devctl_sysf *kysec_devctl_read_sys_devinfo(int *count);

/**
 * @name: kysec_devctl_sync_config
 * @Descripttion: 核外设备同步权限
 * @msg:
 * @param {int} type  类型
 * @param {int} perm 权限
 * @return {*}
 */
int kysec_devctl_auto_sync_config(void *data);

/*获取设备连接记录*/
struct device_record *device_record_get(int *count);

int kysec_devctl_xdg_insert_record(struct device_record *record, int count);

struct kysec_devctl_blacklist *kysec_udisk_read_all_blacklist(int *count);

#ifdef __cplusplus
}
#endif

#endif
